const gulp = require('gulp')
const plugins = require('gulp-load-plugins')()

const sass                      = plugins.sass
    , sourcemaps                = plugins.sourcemaps
    , cssnano                   = plugins.cssnano
    , pug                       = plugins.pug
    , babel                     = plugins.babel
    , autoprefixer              = plugins.autoprefixer
    , uglify                    = plugins.uglify
    , clean                     = plugins.clean
    , changed                   = plugins.changed
    , argv                      = require('yargs').argv
    , gulpif                    = plugins.if
    , imagemin                  = plugins.imagemin
    , less                      = plugins.less
    , plumber                   = plugins.plumber
    , htmlBeautify              = plugins.htmlBeautify
    , htmlmin                   = plugins.htmlmin
    , util                      = plugins.util
    , connectLogger             = require('connect-logger')
    , connectHistoryApiFallback = require('connect-history-api-fallback')
    , serveIndex                = require('serve-index')
    , yod                       = require('yod-mock')
    , chalk                     = require('chalk')
    , mockMiddleware            = require('./vendor/mock/mock-middleware')
    , webpack                   = require('webpack')
    , webpackStream             = require('webpack-stream')
    , path                      = require('path')
    , postcssAutoprefixer       = require('autoprefixer')
    , ExtractTextPlugin         = require('extract-text-webpack-plugin')
    , CommonsChunkPlugin        = require('webpack/lib/optimize/CommonsChunkPlugin')

const app = 'app'
const dest = 'release'
const resource = `${dest}/resource`

const files = {
    css: `${app}/css/**/*.css`
    , sass: `${app}/css/**/*.{sass, scss}`
    , pug: `${app}/view/**/*.pug`
    , babel: `${app}/script/**/*.js`
    , less: `${app}/css/**/*.less`
    , image: `${app}/image/**/*.{jpg,png,svg,gif}`
    , vendor: `${app}/vendor/**/*.*`
}

gulp.task('default', ['serve', 'watch'])

gulp.task('watch', () => {
    gulp.watch(files.css, ['css'])
    gulp.watch(files.sass, ['sass'])
    gulp.watch(files.pug, ['pug'])
    gulp.watch(files.less, ['less'])
    gulp.watch(files.babel, ['babel'])
    gulp.watch(files.image, ['image'])
    gulp.watch(files.vendor, ['vendor'])
    gulp.watch([`${app}/entry/**/*.*`, `./firekeeper/**/*.*`], ['webpack'])
})

gulp.task('css', () => {
    return gulp.src(files.css)
        .pipe(gulpif(argv.d, sourcemaps.init()))
        .pipe(changed(`${resource}/css`))
        .pipe(autoprefixer())
        .pipe(gulpif(!argv.d, cssnano()))
        .pipe(gulpif(argv.d, sourcemaps.write()))
        .pipe(gulp.dest(`${resource}/css`))
})

gulp.task('sass', () => {
    return gulp.src(files.sass)
        .pipe(gulpif(argv.d, sourcemaps.init()))
        .pipe(changed(`${resource}/css`, { extension: 'css' }))
        .pipe(plumber())
        .pipe(sass().on('error', sass.logError))
        .pipe(autoprefixer())
        .pipe(gulpif(!argv.d, cssnano()))
        .pipe(gulpif(argv.d, sourcemaps.write()))
        .pipe(gulp.dest(`${resource}/css`))
})

gulp.task('less', () => {
    return gulp.src(files.less)
        .pipe(gulpif(argv.d, sourcemaps.init()))
        .pipe(changed(`${resource}/css`, { extension: 'css' }))
        .pipe(plumber())
        .pipe(less())
        .pipe(autoprefixer())
        .pipe(gulpif(!argv.d, cssnano()))
        .pipe(gulpif(argv.d, sourcemaps.write()))
        .pipe(gulp.dest(`${resource}/css`))
})

/**
 * 处理图片资源
 */
gulp.task('image', () => {
    return gulp.src(files.image)
        .pipe(changed(`${resource}/image`))
        .pipe(gulpif(!argv.d, imagemin()))
        .pipe(gulp.dest(`${resource}/image`))
})

/**
 * 处理 Pug
 */
gulp.task('pug', () => {
    return gulp.src([files.pug, `!${app}/view/**/_*.pug`])
        .pipe(changed(`${dest}/view`, { extension: 'html' }))
        .pipe(plumber())
        .pipe(pug())
        .pipe(htmlBeautify({
            indent_size: 4
            , indent_char: ' '
            // 设置 head body 标签是否缩进
            , indent_inner_html: false
            , unformatted: true
            // 取消标签前的空行
            , extra_liners: []
        }))
        .pipe(htmlmin({
            // 是否显示单标签斜线
            keepClosingSlash: false
        }))
        .pipe(gulp.dest(`${dest}/view`))
})

/**
 * 处理 vendor 目录
 * 不对 vendor 目录内的内容进行重新打包
 */
gulp.task('vendor', () => {
    return gulp.src(files.vendor)
        .pipe(gulp.dest(`${resource}/vendor`))
})

/**
 * 处理 es6
 */
gulp.task('babel', () => {
    return gulp.src(files.babel)
        .pipe(gulpif(argv.d, sourcemaps.init()))
        .pipe(changed(`${resource}/script`))
        .pipe(babel({ presets: ['es2015'] }))
        .pipe(gulpif(!argv.d, uglify()))
        .pipe(gulpif(argv.d, sourcemaps.write()))
        .pipe(gulp.dest(`${resource}/script`))
})

gulp.task('clean', () => {
    return gulp.src(`${dest}`, { read: false })
        .pipe(clean())
})

gulp.task('release', ['clean'] ,() => {
    gulp.start(['pug', 'sass', 'css', 'babel', 'image', 'vendor', 'webpack'])
})

gulp.task('serve', () => {
    const browserSync = require('browser-sync').create()
    const file = [`${dest}/**/*`]

    browserSync.init(file, {
        open: '',
        notify: false,
        server: { baseDir: dest },
        plugins: ['bs-latency'],
        middleware: [
            mockMiddleware(),
            serveIndex('./release', { icons: true }),
            connectLogger(),
            connectHistoryApiFallback()
        ],
        snippetOptions: {
            rule: {
                match: /<\/head>/i,
                fn: function(snippet, match) {
                    return `${match}${snippet}`
                }
            }
        }
    })
})

gulp.task('webpack', (callback) => {
    return gulp.src('')
        .pipe(webpackStream({
            entry: {
                module: './app/entry/app'
            },
            output: {
                filename: '[name].js'
            },
            module: {
                loaders: [
                    {
                        test: /\.js$/,
                        exclude: [path.resolve('node_modules'), path.resolve('app/vendor')],
                        loaders: ['babel?presets[]=es2015,presets[]=stage-1']
                    },
                    {
                        test: /\.css$/,
                        loaders: ExtractTextPlugin.extract('style', 'css!postcss')
                    },
                    {
                        test: /\.(sass|scss)$/,
                        loader: ExtractTextPlugin.extract('style', 'css!postcss!sass')
                    },
                    {
                        test: /\.less$/,
                        loader: ExtractTextPlugin.extract('style', 'css!postcss!less')
                    },
                    {
                        test: /jquery/,
                        loaders: ['expose?$', 'expose?jQuery']
                    }
                ]
            },
            resolve: {
                alias: {
                    'jquery': 'jquery/dist/jquery.min'
                },
                extensions: ['', '.js']
            },
            plugins: [
                new ExtractTextPlugin('vendor.css'),
                new CommonsChunkPlugin('vendor.js')
            ],
            postcss: function() {
                return [postcssAutoprefixer]
            }
        }))
        .pipe(gulp.dest(`${resource}/vendor`))
})
